@@ -1,4 +1,5 @@
 #include <caml/alloc.h>
+#include <caml/bigarray.h>
 #include <caml/callback.h>
 #include <caml/custom.h>
 #include <caml/fail.h>
@@ -16,7 +17,7 @@ CAMLprim value ocaml_pa_strerror(value e)
 }
 
 /* char* for Some string or NULL for None */
-static char *string_opt_val(value v)
+static const char *string_opt_val(value v)
 {
   if(Is_long(v))
     return NULL;
@@ -147,6 +148,25 @@ CAMLprim value ocaml_pa_simple_write_float(value _simple, value _buf, value _ofs
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_pa_simple_write_float_ba(value _simple, value _buf)
+{
+  CAMLparam2(_simple, _buf);
+  CAMLlocal1(bufc);
+  struct caml_ba_array *ba = Caml_ba_array_val(_buf);
+  int len = ba->dim[0];
+  float *buf = ba->data;
+  int err;
+  int ret;
+  pa_simple *simple = Simple_val(_simple);
+
+  caml_enter_blocking_section();
+  ret = pa_simple_write(simple, buf, len * sizeof(float), &err);
+  caml_leave_blocking_section();
+  check_err(ret, err);
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value ocaml_pa_simple_drain(value _simple)
 {
   CAMLparam1(_simple);
@@ -223,3 +243,22 @@ CAMLprim value ocaml_pa_read_float(value _simple, value _buf, value _ofs, value
 
   CAMLreturn(Val_unit);
 }
+
+CAMLprim value ocaml_pa_read_float_ba(value _simple, value _buf)
+{
+  CAMLparam2(_simple, _buf);
+  pa_simple *simple = Simple_val(_simple);
+  struct caml_ba_array *ba = Caml_ba_array_val(_buf);
+  int buflen = caml_ba_byte_size(ba);
+  float *buf = ba->data;
+  int err, ret;
+
+  caml_enter_blocking_section();
+  ret = pa_simple_read(simple, buf, buflen, &err);
+  caml_leave_blocking_section();
+
+  if(ret < 0)
+    caml_raise_with_arg(*caml_named_value("pa_exn_error"), Val_int(err));
+
+  CAMLreturn(Val_unit);
+}
